#!/usr/bin/perl
# Author: Mike Brodbelt
# Date: 31/07/2006
# Last update: 4/08/2006

# This is an ugly hack necessitated by the fact that several devices around have 64 bit
# counters, but will only offer SNMP v1, which is limited to 32 bit return values from OID
# queries. We use Bit::Vector to deal with integers that won't fit in 32 bits.

use Bit::Vector;

my $snmpget = "/usr/local/bin/snmpget";
my $snmptable = "/usr/local/bin/snmptable";

my $nodetableoid = ".1.3.6.1.4.1.7146.1.2.4.2";		# ZXTM-MIB::nodeTable
my $vsertableoid = ".1.3.6.1.4.1.7146.1.2.2.2";		# ZXTM-MIB::virtualserverTable

my $usage = "usage:\n\n" .
	    "$0 {nodes|vservers} <snmp_community> <zxtm_ip> index\n" .
	    "$0 nodes <snmp_community> <zxtm_ip> query {ip|port|host|state|bytestonode|bytesfromnode|conns|pool_conns|failures|new_conns}\n" .
	    "$0 nodes <snmp_community> <zxtm_ip> get {ip|port|host|state|bytestonode|bytesfromnode|conns|pool_conns|failures|new_conns} NODE\n" .
	    "$0 vservers <snmp_community> <zxtm_ip> query {name|port|protocol|trafficpool|cnn|btovsrv|bfrmvsrv|dscd|dtrp|cntmt|dttmt|kplvtmt|udptmt|udpdgm|gz|gzbsvd|httprwloc|httprwck|httpcchhit|httpcchlkup}\n" .
	    "$0 vservers <snmp_community> <zxtm_ip> get {name|port|protocol|trafficpool|cnn|btovsrv|bfrmvsrv|dscd|dtrp|cntmt|dttmt|kplvtmt|udptmt|udpdgm|gz|gzbsvd|httprwloc|httprwck|httpcchhit|httpcchlkup} VSERVER\n";



scalar(@ARGV) >= "4" or die "$usage";
my ($querytype, $community, $address, $action, $req_out, $req_index) = @ARGV;
my (%output,@fields);

# Sanity check input
if (($action eq "query") && (!$req_out)) { die "$usage" }
if (($action eq "get") && (!(($req_out) && ($req_index)))) { die "$usage" }

if ($querytype eq "nodes") {
	if ($action eq "index") {
		open(NODELIST, "$snmptable -CHif : -v 1 -c $community $address $nodetableoid | ");
		foreach (<NODELIST>) { chomp; ($index) = split(/:/, $_); print "$index\n"; }
	} elsif (($action eq "query") || ($action eq "get")) {
		open(NODELIST, "$snmptable -CHif : -v 1 -c $community $address $nodetableoid | ");
		foreach (<NODELIST>) {
			chomp;
			if ($action eq "get") {				# If we're only getting data for a single row,
				next unless /^${req_index}:/;		# don't bother parsing the ones that don't match.
			}
			@fields = split(/:/, $_);
			my %output = (
				index => $fields[0],
				ip => $fields[1],
				port => $fields[2],
				host => $fields[3],
				state => $fields[4],
				conns => $fields[10],
				pool_conns => $fields[11],
				failures => $fields[12],
				new_conns => $fields[13]
			);

			if ($ARGV[4] =~ /^bytes(to|from)node$/) {	# We only do the binary arithmetic if we actually need to
				$output{bytestonode} = ((Bit::Vector->new_Dec(33, $fields[6]))->
								Concat(Bit::Vector->new_Dec(32, $fields[5])))->to_Dec();
				$output{bytesfromnode} = ((Bit::Vector->new_Dec(33, $fields[8]))->
								Concat(Bit::Vector->new_Dec(32, $fields[7])))->to_Dec();
			}
			($action eq "query") ? print "$output{index}:$output{$ARGV[4]}\n" : print "$output{$ARGV[4]}\n";
		}
	} else {
		print "Invalid action: $action\n";
	}
} elsif ($querytype eq "vservers") {
	if ($action eq "index") {
		open(VSERVERLIST, "$snmptable -CHif : -v 1 -c $community $address $vsertableoid | ");
		foreach (<VSERVERLIST>) { chomp; s/"//g; ($index) = split(/:/, $_); print "$index\n"; }
	} elsif (($action eq "query") || ($action eq "get")) {
		open(VSERVERLIST, "$snmptable -CHif : -v 1 -c $community $address $vsertableoid | ");
		foreach (<VSERVERLIST>) {
			chomp;
			s/"//g;
			if ($action eq "get") {				# If we're only getting data for a single row,
				next unless /^${req_index}:/;		# don't bother parsing the ones that don't match.
			}
			@fields = split(/:/, $_);
			my %output = (
				name => $fields[1],			# We use this field as the index
				port => $fields[2],
				protocol => $fields[3],
				trafficpool => $fields[4],
				cnn => $fields[11],
				dscd => $fields[12],
				dtrp => $fields[13],
				cntmt => $fields[14],
				dttmt => $fields[15],
				kplvtmt => $fields[16],
				udptmt => $fields[17],
				udpdgm => $fields[18],
				gz => $fields[19],
				httprwloc => $fields[22],
				httprwck => $fields[23],
				httpcchhit => $fields[24],
				httpcchlkup => $fields[25]
			);
			if ($ARGV[4] =~ /^b(to|frm)vsrv$/) {	# We only do the binary arithmetic if we actually need to
				$output{btovsrv} = ((Bit::Vector->new_Dec(33, $fields[6]))->
								Concat(Bit::Vector->new_Dec(32, $fields[5])))->to_Dec();
				$output{bfrmvsrv} = ((Bit::Vector->new_Dec(33, $fields[8]))->
								Concat(Bit::Vector->new_Dec(32, $fields[7])))->to_Dec();
			}
			if ($ARGV[4] =~ /^gzbsvd$/) {	# We only do the binary arithmetic if we actually need to
				$output{gzbsvd} = ((Bit::Vector->new_Dec(33, $fields[21]))->
								Concat(Bit::Vector->new_Dec(32, $fields[20])))->to_Dec();
			}
			($action eq "query") ? print "$output{name}:$output{$ARGV[4]}\n" : print "$output{$ARGV[4]}\n";
				
		}
	} else {
		print "Invalid action: $action\n";
	}
} else {
	print "Invalid query type: $querytype\n";
}
